@@ -93,7 +93,7 @@ gboolean                g_vfs_ftp_task_login                    (GVfsFtpTask *
                                                                  const char *           password);
 void                    g_vfs_ftp_task_setup_connection         (GVfsFtpTask *          task);
 
-gboolean                g_vfs_ftp_task_enable_tls               (GVfsFtpTask *          task,
+gboolean                g_vfs_ftp_task_initial_handshake        (GVfsFtpTask *          task,
                                                                  CertificateCallback    cb,
                                                                  gpointer               user_data);
 
